pub struct BasicBlock {
pub attributes: AttributeDict,
/* private fields */
}Expand description
A basic block contains a list of Operations. It may have arguments.
Fields
attributes: AttributeDictA dictionary of attributes.
Implementations
Sourceimpl BasicBlock
impl BasicBlock
Sourcepub fn new(
ctx: &mut Context,
label: Option<Identifier>,
arg_types: Vec<Ptr<TypeObj>>,
) -> Ptr<BasicBlock>
pub fn new( ctx: &mut Context, label: Option<Identifier>, arg_types: Vec<Ptr<TypeObj>>, ) -> Ptr<BasicBlock>
Create a new Basic Block.
Sourcepub fn get_parent_region(&self) -> Option<Ptr<Region>>
pub fn get_parent_region(&self) -> Option<Ptr<Region>>
Get parent region.
Sourcepub fn get_parent_block(&self, ctx: &Context) -> Option<Ptr<BasicBlock>>
pub fn get_parent_block(&self, ctx: &Context) -> Option<Ptr<BasicBlock>>
Get parent block.
Sourcepub fn get_argument(&self, arg_idx: usize) -> Value
pub fn get_argument(&self, arg_idx: usize) -> Value
Get idx’th argument as a Value. Panics on invalid index.
Sourcepub fn push_argument(
block: Ptr<BasicBlock>,
ctx: &Context,
ty: Ptr<TypeObj>,
) -> usize
pub fn push_argument( block: Ptr<BasicBlock>, ctx: &Context, ty: Ptr<TypeObj>, ) -> usize
Add an argument to the end of the argument list, returning its index.
Sourcepub fn pop_argument(block: Ptr<BasicBlock>, ctx: &Context)
pub fn pop_argument(block: Ptr<BasicBlock>, ctx: &Context)
Remove the last argument. Panics if there are no arguments or if the argument has uses. Any Value referring to the removed argument is invalidated.
Sourcepub fn insert_argument(
block: Ptr<BasicBlock>,
ctx: &Context,
arg_idx: usize,
ty: Ptr<TypeObj>,
)
pub fn insert_argument( block: Ptr<BasicBlock>, ctx: &Context, arg_idx: usize, ty: Ptr<TypeObj>, )
Insert a new argument at arg_idx, shifting existing arguments, from arg_idx, to the right.
Panics on invalid index.
Sourcepub fn remove_argument(block: Ptr<BasicBlock>, ctx: &Context, arg_idx: usize)
pub fn remove_argument(block: Ptr<BasicBlock>, ctx: &Context, arg_idx: usize)
Remove the argument at arg_idx, shifting subsequent arguments to the left.
Panics on invalid index or if the argument has uses.
Any Value referring to the removed argument is invalidated.
Sourcepub fn get_num_arguments(&self) -> usize
pub fn get_num_arguments(&self) -> usize
Get the number of arguments.
Sourcepub fn has_succ(&self, ctx: &Context, succ: Ptr<BasicBlock>) -> bool
pub fn has_succ(&self, ctx: &Context, succ: Ptr<BasicBlock>) -> bool
Is succ a successor of this block?
Sourcepub fn get_terminator(&self, ctx: &Context) -> Option<Ptr<Operation>>
pub fn get_terminator(&self, ctx: &Context) -> Option<Ptr<Operation>>
Get the block terminator, if one exists.
Sourcepub fn drop_all_uses(ptr: Ptr<Self>, ctx: &Context)
pub fn drop_all_uses(ptr: Ptr<Self>, ctx: &Context)
Drop all uses that this block holds.