SymbolTable

Struct SymbolTable 

pub struct SymbolTable { /* private fields */ }
A utility to efficiently lookup and update Symbols in a SymbolTableInterface Op. Similar to its counterpart in MLIR, inserting into and erasing from this SymbolTable will also insert and erase from the Operation given to it at construction.

Implementations

Source

impl SymbolTable

pub fn new( ctx: &Context, symbol_table_op: Box<dyn SymbolTableInterface>, ) -> Self

Create a new SymbolTable from a SymbolTableInterface Op.

pub fn insert( &mut self, ctx: &Context, symbol_op: Box<dyn SymbolOpInterface>, insert_pt: Option<OpInsertionPoint>, ) -> Result<()>

Insert a new Symbol into the SymbolTable. If an insertion point is provided, that point must be inside self.symbol_table_op. If no insertion point is provided, the symbol op will be inserted at the end (before the terminator if one exists). symbol_op’s parent must be either None, or the same as self.symbol_table_op.

pub fn remove( &mut self, ctx: &Context, symbol_op: Box<dyn SymbolOpInterface>, ) -> Result<()>

Remove symbol_op from this SymbolTable. The operation is unlinked from its parent, but is not erased.

pub fn erase( &mut self, ctx: &mut Context, symbol_op: Box<dyn SymbolOpInterface>, ) -> Result<()>

Remove symbol_op from this SymbolTable and erase it. This will panic if symbol_op has SSA uses.

pub fn lookup( &self, symbol_name: &Identifier, ) -> Option<Box<dyn SymbolOpInterface>>

Get the Symbol with the given name.